Program Fair;

{$mode objfpc}

Uses
  SysUtils;

type
  TDigit = 0..9;
  TDigitSet = set of TDigit;

const
  LowerLimit = Low(SmallInt);
  UpperLimit = High(SmallInt);

  ListHeader = 'Listing all "fair" numbers from %d to %d';
  FoundFairNumber = 'Found a fair number: %d';
  AskLowerLiMit = 'Enter value for lower limit [%d..%d]: ';
  AskUpperLimit = 'Enter value for upper limit [%d..%d]: ';


{$ifdef debug}
function DbgS(Digits: TDigitSet): String;
var
  D: TDigit;
begin
  Result := '[';
  for D := Low(TDigit) to High(TDigit) do
    if D in Digits then
      Result := Result + IntToStr(D) + ',';
  if (Result[Length(Result)] = ',') then
    System.Delete(Result, Length(Result), 1);
  Result := Result + ']';
end;
{$endif}

function ExtractDigits(const ANr: Integer): TDigitSet;
const
  AllDigits: TDigitSet = [0,1,2,3,4,5,6,7,8,9];
var
  TempNr: Integer;
  ADigit: TDigit;
begin
  Result := [];
  TempNr := Abs(ANr);
  while (TempNr <> 0) and not ((AllDigits * Result) = AllDigits) do
  begin
    ADigit := TempNr mod 10;
    Result := Result + [ADigit];
    TempNr := TempNr div 10;
  end;
end;

function IsFair(const ANr: Integer): Boolean;
var
  ADigit: TDigit;
  Digits: TDigitSet;
begin
  Result := False;
  if (ANr = 0) then Exit; //shortcut
  Digits := ExtractDigits(ANr);
  {$ifdef debug}
  writeln(format('ANr = %d, Digits = %s',[ANr,DbgS(Digits)]));
  {$endif}
  for ADigit := Low(TDigit) to High(TDigit) do
  begin
    if ADigit in Digits then
      if (ADigit = 0) or ((ANr mod ADigit) <> 0) then Exit; //False
  end;
  Result := True;
end;

procedure ListFairNumbers(const Start, Stop: Integer);
var
  i: Integer;
begin
  writeln(Format(ListHeader,[Start, Stop]));
  for i := Start to Stop do
    if IsFair(i) then writeln(Format(FoundFairNumber,[i]));
end;

procedure GetLimits(out Start, Stop: Integer);
//either accept commandline parameters or ask for the limits
var
  HaveStart, HaveStop: Boolean;
  val: String;
  Temp: Integer;
begin
  {$ifdef debug}
  Start := 10;
  Stop := 256;
  Exit;
  {$endif debug}
  HaveStart := False;
  HaveStop := False;
  if (ParamCount > 0) and TryStrToInt(ParamStr(1), Start) and (Start >= LowerLimit) and (Start <= UpperLimit) then
  begin
    HaveStart := True;
    if (ParamCount > 1) and TryStrToInt(ParamStr(2), Stop) and (Stop >= LowerLimit) and (Stop <= UpperLimit) then
      HaveStop := True;
  end;
  while not HaveStart do
  begin
    write(Format(AskLowerLimit,[LowerLimit, UpperLimit]));
    readln(val);
    HaveStart := TryStrToInt(val, Start)  and (Start >= LowerLimit) and (Start <= UpperLimit);
  end;
  while not HaveStop do
  begin
    write(Format(AskUpperLimit,[LowerLimit, UpperLimit]));
    readln(val);
    HaveStop := TryStrToInt(val, Stop) and (Stop >= LowerLimit) and (Stop <= UpperLimit);
  end;
  if (Start > Stop) then
  begin
    Temp := Start;
    Start := Stop;
    Stop := Temp;
  end;
end;

var
  Start, Stop: Integer;

begin
  GetLimits(Start, Stop);
  ListFairNumbers(Start, Stop);
end.

